@@ -118,14 +118,6 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 
 #endif	/* !_MSC_VER */
 
-#ifdef HAVE_STRLCPY
-#define strncpy strlcpy
-#endif
-
-#ifdef HAVE_STRLCAT
-#define strncat strlcat
-#endif
-
 #ifdef _MSC_VER
 #define dup _dup
 #define fileno _fileno
@@ -170,6 +162,7 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 #define QUIRK_ENVSUS	(1 << 27)	/* Key release jumps to sus point */
 #define QUIRK_S3MPMEM	(1 << 28)	/* S3M-style parameter memory */
 #define QUIRK_XMFINE	(1 << 29)	/* XM-style fine tune */
+#define QUIRK_RSTCHN	(1 << 30)	/* Reset channel on sample end */
 
 #define HAS_QUIRK(x)	(m->quirk & (x))
 
@@ -182,7 +175,7 @@ void __inline CLIB_DECL D_(const char *text, ...) { do {} while (0); }
 #define QUIRKS_IT		(QUIRK_S3MLOOP | QUIRK_FINEFX | QUIRK_VIBALL | \
 				 QUIRK_ENVFADE | QUIRK_ITVPOR | QUIRK_KEYOFF | \
 				 QUIRK_VIRTUAL | QUIRK_FILTER | QUIRK_S3MLFO | \
-				 QUIRK_IGSTPOR | QUIRK_S3MRTG )
+				 QUIRK_IGSTPOR | QUIRK_S3MRTG | QUIRK_RSTCHN )
 
 /* DSP effects */
 #define DSP_EFFECT_CUTOFF	0x02
@@ -203,6 +196,9 @@ struct ord_data {
 	int gvl;
 	int time;
 	int start_row;
+#ifndef LIBXMP_CORE_PLAYER
+	int st26_speed;
+#endif
 };
 
 
@@ -329,6 +325,11 @@ struct player_data {
 		int in_size;
 		char *in_buffer;
 	} buffer_data;
+
+#ifndef LIBXMP_CORE_PLAYER
+	int st26_speed;			/* For IceTracker speed effect */
+#endif
+
 };
 
 struct mixer_data {
@@ -386,11 +387,6 @@ uint32	readmem24b		(uint8 *);
 uint32	readmem32l		(uint8 *);
 uint32	readmem32b		(uint8 *);
 
-int	get_temp_dir		(char *, int);
-#ifdef WIN32
-int	mkstemp			(char *);
-#endif
-
 struct xmp_instrument *get_instrument(struct context_data *, int);
 struct xmp_sample *get_sample(struct context_data *, int);
 
