@@ -9,10 +9,23 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include "common.h"
+#include "depacker.h"
 #include "inflate.h"
 
+static int test_muse(unsigned char *b)
+{
+	if (memcmp(b, "MUSE", 4) == 0) {
+		uint32 r = readmem32b(b + 4);
+		/* MOD2J2B uses 0xdeadbabe */
+		if (r == 0xdeadbeaf || r == 0xdeadbabe) {
+			return 1;
+		}
+	}
 
-int decrunch_muse(FILE *f, FILE *fo)                          
+	return 0;
+}
+
+static int decrunch_muse(FILE *f, FILE *fo)                          
 {                                                          
 	uint32 checksum;
   
@@ -21,3 +34,8 @@ int decrunch_muse(FILE *f, FILE *fo)
 
 	return 0;
 }
+
+struct depacker muse_depacker = {
+	test_muse,
+	decrunch_muse
+};
