@@ -1,4 +1,4 @@
-/* Extended Module Player core player
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -82,14 +82,24 @@ static void do_toneporta(struct module_data *m,
 
 
 void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
-		uint8 note, uint8 fxt, uint8 fxp, int fnum)
+		struct xmp_event *e, int fnum)
 {
 	struct player_data *p = &ctx->p;
 	struct module_data *m = &ctx->m;
 	struct xmp_module *mod = &m->mod;
 	struct flow_control *f = &p->flow;
+	uint8 note, fxp, fxt;
 	int h, l;
 
+	note = e->note;
+	if (fnum == 0) {
+		fxt = e->fxt;
+		fxp = e->fxp;
+	} else {
+		fxt = e->f2t;
+		fxp = e->f2p;
+	}
+
 	switch (fxt) {
 	case FX_ARPEGGIO:
 	      fx_arpeggio:
@@ -462,8 +472,12 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 	case FX_S3M_SPEED:	/* Set S3M speed */
 		EFFECT_MEMORY_S3M(fxp);
 	    fx_s3m_speed:
-		if (fxp)
+		if (fxp) {
 			p->speed = fxp;
+#ifndef LIBXMP_CORE_PLAYER
+			p->st26_speed = 0;
+#endif
+		}
 		break;
 	case FX_S3M_BPM:	/* Set S3M BPM */
             fx_s3m_bpm:
@@ -696,6 +710,18 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 #endif
 
 #ifndef LIBXMP_CORE_PLAYER
+	/* Saga Musix says:
+	 *
+	 * "When both nibbles of an Fxx command are set, SoundTracker 2.6
+	 * applies the both values alternatingly, first the high nibble,
+	 * then the low nibble on the next row, then the high nibble again...
+	 * If only the high nibble is set, it should act like if only the low
+	 * nibble is set (i.e. F30 is the same as F03).
+	 */
+	case FX_ICE_SPEED:
+		p->st26_speed = (MSN(fxp) << 8) | LSN(fxp);
+		break;
+
 	case FX_VOLSLIDE_UP:	/* Vol slide with uint8 arg */
 		if (HAS_QUIRK(QUIRK_FINEFX)) {
 			h = MSN(fxp);
@@ -824,8 +850,12 @@ void process_fx(struct context_data *ctx, struct channel_data *xc, int chn,
 		SET_LFO_NOTZERO(&xc->vibrato.lfo, LSN(fxp) << 3, MSN(fxp));
 		break;
 	case FX_SPEED_CP:	/* Set speed and ... */
-		if (fxp)
+		if (fxp) {
 			p->speed = fxp;
+#ifndef LIBXMP_CORE_PLAYER
+			p->st26_speed = 0;
+#endif
+		}
 		/* fall through */
 	case FX_PER_CANCEL:	/* Cancel persistent effects */
 		xc->per_flags = 0;
