@@ -1,4 +1,4 @@
-/* Extended Module Player core player
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -25,22 +25,39 @@
 
 /* Envelope */
 
-int get_envelope(struct xmp_envelope *env, int x, int def, int *end)
+int check_envelope_end(struct xmp_envelope *env, int x)
+{
+	int16 *data = env->data;
+	int index;
+
+	if (~env->flg & XMP_ENVELOPE_ON || env->npt <= 0)
+		return 0;
+
+	index = (env->npt - 1) * 2;
+
+	/* last node */
+	if (x >= data[index] || index == 0) { 
+		if (~env->flg & XMP_ENVELOPE_LOOP) {
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+int get_envelope(struct xmp_envelope *env, int x, int def)
 {
 	int x1, x2, y1, y2;
 	int16 *data = env->data;
 	int index;
 
-	*end = 0;
-
-	if (~env->flg & XMP_ENVELOPE_ON || env->npt <= 0)
+	if (x < 0 || ~env->flg & XMP_ENVELOPE_ON || env->npt <= 0)
 		return def;
 
 	index = (env->npt - 1) * 2;
 
 	x1 = data[index];		/* last node */
 	if (x >= x1 || index == 0) { 
-		*end = 1;
 		return data[index + 1];
 	}
 
@@ -69,6 +86,9 @@ int update_envelope(struct xmp_envelope *env, int x, int release, int sus_quirk)
 	int has_loop, has_sus;
 	int lpe, lps, sus, sue;
 
+	if (x < 0)
+		return -1;
+
 	if (~env->flg & XMP_ENVELOPE_ON || env->npt <= 0) {
 		return x;
 	}
