@@ -3,8 +3,9 @@
 
 /* Envelope */
 
-int get_envelope(struct xmp_envelope *, int, int, int *);
+int get_envelope(struct xmp_envelope *, int, int);
 int update_envelope(struct xmp_envelope *, int, int, int);
 int check_envelope_fade(struct xmp_envelope *, int);
+int check_envelope_end(struct xmp_envelope *, int);
 
 #endif
