@@ -1,11 +1,24 @@
 /* Extended Module Player
  * AMOS/STOS Music Bank Loader
- *
  * Copyright (C) 2014 Stephen J Leary and Claudio Matsuoka
  *
- * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU General Public License. See doc/COPYING
- * for more information.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -20,7 +33,6 @@
 
 #define AMOS_BANK 0x416d426b
 #define AMOS_MUSIC_TYPE 0x0003
-#define AMOS_MUSIC_TEXT 0x4D75736963202020
 #define AMOS_MAIN_HEADER 0x14L
 #define AMOS_STRING_LEN 0x10
 #define AMOS_BASE_FREQ 8192
@@ -87,17 +99,6 @@ struct abk_instrument
 };
 
 
-static void free_abk_playlist(struct abk_playlist *playlist)
-{
-    if (playlist->pattern != NULL)
-    {
-        free(playlist->pattern);
-    }
-
-    playlist->length = 0;
-}
-
-
 /**
  * @brief read the ABK playlist out from the file stream. This method malloc's some memory for the playlist
  * and can realloc if the playlist is very long.
@@ -401,7 +402,7 @@ static struct abk_instrument* read_abk_insts(HIO_HANDLE *f, uint32 inst_section_
 
 static int abk_test(HIO_HANDLE *f, char *t, const int start)
 {
-    uint64 music;
+    char music[8];
 
     if (hio_read32b(f) != AMOS_BANK)
     {
@@ -416,11 +417,9 @@ static int abk_test(HIO_HANDLE *f, char *t, const int start)
     /* skip over length and chip/fastmem.*/
     hio_seek(f, 6, SEEK_CUR);
 
-    music = hio_read32b(f); /* get the "Music   " */
-    music = music << 32;
-    music |= hio_read32b(f);
+    hio_read(music, 1, 8, f);	/* get the "Music   " */
 
-    if (music != (unsigned long long)AMOS_MUSIC_TEXT)
+    if (memcmp(music, "Music   ", 8))
     {
         return -1;
     }
@@ -434,7 +433,6 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
     uint16 pattern;
     uint32 first_sample_offset;
     uint32 inst_section_size;
-    uint32 file_size;
 
     struct xmp_module *mod = &m->mod;
 
@@ -443,13 +441,6 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
     struct abk_song song;
     struct abk_playlist playlist;
 
-    ci = NULL;
-    pattern = 0;
-    first_sample_offset = 0;
-
-    hio_seek(f, 0, SEEK_END);
-    file_size = hio_tell(f);
-
     hio_seek(f, AMOS_MAIN_HEADER, SEEK_SET);
 
     main_header.instruments_offset = hio_read32b(f);
@@ -502,6 +493,7 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
     {
         if (subinstrument_alloc(mod, i, 1) < 0)
         {
+            free(ci);
             return -1;
         }
 
@@ -539,6 +531,8 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
            mod->xxs[i].flg & XMP_SAMPLE_LOOP ? 'L' : ' ');
     }
 
+    free(ci);
+
     if (pattern_init(mod) < 0)
     {
         return -1;
@@ -561,6 +555,7 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
     {
         if (pattern_tracks_alloc(mod, i, 64) < 0)
         {
+            free(playlist.pattern);
             return -1;
         }
 
@@ -584,8 +579,7 @@ static int abk_load(struct module_data *m, HIO_HANDLE *f, const int start)
     }
 
     /* free up some memory here */
-    free(ci);
-    free_abk_playlist(&playlist);
+    free(playlist.pattern);
 
     D_(D_INFO "Stored patterns: %d", mod->pat);
     D_(D_INFO "Stored tracks: %d", mod->trk);
