@@ -1,9 +1,23 @@
 /* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
- * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU Lesser General Public License. See COPYING.LIB
- * for more information.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
 /* Based on DigiBooster_E.guide from the DigiBoosterPro 2.20 package.
@@ -152,7 +166,7 @@ static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 		//printf("rows = %d, size = %d\n", mod->xxp[i]->rows, sz);
 
 		r = 0;
-		c = -1;
+		/*c = -1;*/
 
 		while (sz > 0) {
 			//printf("  offset=%x,  sz = %d, ", hio_tell(f), sz);
@@ -162,7 +176,6 @@ static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 
 			if (c == 0) {
 				r++;
-				c = -1;
 				continue;
 			}
 			c--;
@@ -171,10 +184,13 @@ static int get_patt(struct module_data *m, int size, HIO_HANDLE *f, void *parm)
 			if (--sz <= 0) break;
 			//printf("    n = %d\n", n);
 
-			if (c >= mod->chn || r >= mod->xxp[i]->rows)
+			if (c >= mod->chn || r >= mod->xxp[i]->rows) {
 				event = &dummy;
-			else
+			} else {
 				event = &EVENT(i, c, r);
+			}
+
+			memset(event, 0, sizeof (struct xmp_event));
 
 			if (n & 0x01) {
 				x = hio_read8(f);
