@@ -1,13 +1,25 @@
 /* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
- * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU Lesser General Public License. See COPYING.LIB
- * for more information.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
-#include <sys/types.h>
-#include <sys/stat.h>
 #include "loader.h"
 
 #define MAGIC_Funk	MAGIC4('F','u','n','k')
@@ -26,7 +38,6 @@ static int fnk_test(HIO_HANDLE *f, char *t, const int start)
 {
     uint8 a, b;
     int size;
-    struct stat st;
 
     if (hio_read32b(f) != MAGIC_Funk)
 	return -1;
@@ -46,8 +57,7 @@ static int fnk_test(HIO_HANDLE *f, char *t, const int start)
     if (size < 1024)
 	return -1;
 
-    hio_stat(f, &st);
-    if (size != st.st_size)
+    if (hio_size(f) != size)
         return -1;
 
     read_title(f, t, 0);
