@@ -1,4 +1,4 @@
-/* Extended Module Player format loaders
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -229,7 +229,7 @@ static int mmd3_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	expdata.i_ext_entrsz = 0;
 	expsmp_offset = 0;
 	iinfo_offset = 0;
-	mmdinfo_offset = 0;
+
 	if (expdata_offset) {
 		hio_seek(f, start + expdata_offset, SEEK_SET);
 		hio_read32b(f);
