@@ -1,4 +1,4 @@
-/* Extended Module Player format loaders
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -36,9 +36,6 @@
  */
 
 #include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
 #include <limits.h>
 #include "loader.h"
 #include "mod.h"
@@ -96,12 +93,28 @@ const struct format_loader mod_loader = {
     mod_load
 };
 
+static int validate_pattern(uint8 *buf)
+{
+	int i, j;
+
+	for (i = 0; i < 64; i++) {
+		for (j = 0; j < 4; j++) {
+			uint8 *d = buf + (i * 4 + j) * 4;
+			if ((d[0] >> 4) > 1)
+				return -1;
+		}
+	}
+
+	return 0;
+}
+
 static int mod_test(HIO_HANDLE *f, char *t, const int start)
 {
     int i;
     char buf[4];
-    struct stat st;
+    uint8 pat_buf[1024];
     int smp_size, num_pat;
+    long size;
 
     hio_seek(f, start + 1080, SEEK_SET);
     if (hio_read(buf, 1, 4, f) < 4)
@@ -147,9 +160,6 @@ static int mod_test(HIO_HANDLE *f, char *t, const int start)
 		return -1;
     }
 
-    if (HIO_HANDLE_TYPE(f) != HIO_HANDLE_TYPE_FILE)
-	goto found;
-
     /* Test for UNIC tracker modules
      *
      * From Gryzor's Pro-Wizard PW_FORMATS-Engl.guide:
@@ -160,7 +170,7 @@ static int mod_test(HIO_HANDLE *f, char *t, const int start)
      */
 
     /* get file size */
-    hio_stat(f, &st);
+    size = hio_size(f);
     smp_size = 0;
     hio_seek(f, start + 20, SEEK_SET);
 
@@ -183,9 +193,18 @@ static int mod_test(HIO_HANDLE *f, char *t, const int start)
     }
     num_pat++;
 
-    if (start + 1084 + num_pat * 0x300 + smp_size == st.st_size)
+    /* see if module size matches UNIC */
+    if (start + 1084 + num_pat * 0x300 + smp_size == size)
 	return -1;
 
+    /* validate pattern data in an attempt to catch UNICs with MOD size */
+    for (i = 0; i < num_pat; i++) {
+	hio_seek(f, start + 1084 + 1024 * i, SEEK_SET);
+	hio_read(pat_buf, 1024, 1, f);
+	if (validate_pattern(pat_buf) < 0)
+	    return -1;
+    }
+
   found:
     hio_seek(f, start + 0, SEEK_SET);
     read_title(f, t, 20);
@@ -222,6 +241,7 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
     char magic[8], idbuffer[32];
     int ptkloop = 0;			/* Protracker loop */
     int tracker_id = TRACKER_PROTRACKER;
+    int has_loop_0 = 0;
 
     LOAD_INIT();
 
@@ -369,6 +389,15 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	goto skip_test;
     }
 
+    /* Check if has instruments with loop size 0 */
+    for (i = 0; i < 31; i++) {
+	if (mh.ins[i].loop_size == 0)
+	    break;
+    }
+    if (i < 31) {
+        has_loop_0 = 1;
+    }
+
     /* Test Protracker-like files
      */
     if (mod->chn == 4 && mh.restart == mod->pat) {
@@ -392,21 +421,13 @@ static int mod_load(struct module_data *m, HIO_HANDLE *f, const int start)
     }
 
     if (mod->chn == 4 && mh.restart == 0x7f) {
-	for (i = 0; i < 31; i++) {
-	    if (mh.ins[i].loop_size == 0)
-		break;
-	}
-	if (i < 31) {
+	if (has_loop_0) {
 	    tracker_id = TRACKER_CLONE;
 	}
     }
 
     if (mh.restart != 0x78 && mh.restart < 0x7f) {
-	for (i = 0; i < 31; i++) {
-	    if (mh.ins[i].loop_size == 0)
-		break;
-	}
-	if (i == 31) {	/* All loops are size 2 or greater */
+	if (!has_loop_0) {	/* All loops are size 2 or greater */
 	    for (i = 0; i < 31; i++) {
 		if (mh.ins[i].size == 1 && mh.ins[i].volume == 0) {
 		    tracker_id = TRACKER_CONVERTED;
@@ -499,16 +520,30 @@ skip_test:
 	    return -1;
 
 	for (j = 0; j < (64 * mod->chn); j++) {
+            int period;
+
 	    event = &EVENT (i, j % mod->chn, j / mod->chn);
 	    hio_read (mod_event, 1, 4, f);
 
+	    /* Check out-of-range notes in Amiga trackers */
+	    period = ((int)(LSN(mod_event[0])) << 8) | mod_event[1];
+	    if (period != 0 && period < 108 && (
+			tracker_id == TRACKER_PROTRACKER ||
+			tracker_id == TRACKER_NOISETRACKER ||
+			tracker_id == TRACKER_PROBABLY_NOISETRACKER ||
+			tracker_id == TRACKER_SOUNDTRACKER)) {	/* note > B-3 */
+		tracker_id = TRACKER_UNKNOWN;
+	    }
+
 	    /* Filter noisetracker events */
 	    if (tracker_id == TRACKER_PROBABLY_NOISETRACKER) {
 		unsigned char fxt = LSN(mod_event[2]);
 		unsigned char fxp = LSN(mod_event[3]);
-        	if ((fxt > 0x06 && fxt < 0x0a) ||  (fxt == 0x0e && fxp > 1)) {
+
+        	if ((fxt > 0x06 && fxt < 0x0a) || (fxt == 0x0e && fxp > 1)) {
 		    tracker_id = TRACKER_UNKNOWN;
 		}
+
 	    }
 
 	    switch (tracker_id) {
