@@ -1,10 +1,11 @@
 /*
  * ac1d.c   Copyright (C) 1996-1997 Asle / ReDoX
- *	    Copyright (C) 2006-2007 Claudio Matsuoka
  *
- * Converts AC1D packed MODs back to PTK MODs
+ * Converts ac1d packed MODs back to PTK MODs
  * thanks to Gryzor and his ProWizard tool ! ... without it, this prog
  * would not exist !!!
+ *
+ * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
 #include <stdlib.h>
@@ -13,7 +14,7 @@
 
 #define NO_NOTE 0xff
 
-static int depack_AC1D(FILE *in, FILE *out)
+static int depack_ac1d(FILE *in, FILE *out)
 {
 	uint8 c1, c2, c3, c4;
 	uint8 npos;
@@ -26,7 +27,7 @@ static int depack_AC1D(FILE *in, FILE *out)
 	int ssize = 0;
 	int paddr[128];
 	int psize[128];
-	int tsize1, tsize2, tsize3;
+	/*int tsize1, tsize2, tsize3;*/
 	int i, j, k;
 
 	memset(paddr, 0, 128 * 4);
@@ -71,16 +72,15 @@ static int depack_AC1D(FILE *in, FILE *out)
 	/* pattern data */
 	for (i = 0; i < npat; i++) {
 		fseek(in, paddr[i], SEEK_SET);
-		tsize1 = read32b(in);
-		tsize2 = read32b(in);
-		tsize3 = read32b(in);
+		/*tsize1 =*/ read32b(in);
+		/*tsize2 =*/ read32b(in);
+		/*tsize3 =*/ read32b(in);
 
 		memset(tmp, 0, 1024);
 		for (k = 0; k < 4; k++) {
 			for (j = 0; j < 64; j++) {
 				int x = j * 16 + k * 4;
 
-				note = ins = fxt = fxp = 0x00;
 				c1 = read8(in);
 				if (c1 & 0x80) {
 					c4 = c1 & 0x7f;
@@ -108,8 +108,6 @@ static int depack_AC1D(FILE *in, FILE *out)
 				}
 
 				if ((c2 & 0x0f) == 0x07) {
-					fxt = 0x00;
-					fxp = 0x00;
 					tmp[x + 2] = (ins << 4) & 0xf0;
 					continue;
 				}
@@ -131,10 +129,9 @@ static int depack_AC1D(FILE *in, FILE *out)
 	return 0;
 }
 
-static int test_AC1D(uint8 *data, char *t, int s)
+static int test_ac1d(uint8 *data, char *t, int s)
 {
-	int j, k;
-	int start = 0;
+	int i;
 
 	PW_REQUEST_DATA(s, 896);
 
@@ -143,18 +140,18 @@ static int test_AC1D(uint8 *data, char *t, int s)
 		return -1;
 
 	/* test #2 */
-	if (data[start] > 0x7f)
+	if (data[0] > 0x7f)
 		return -1;
 
 	/* test #4 */
-	for (k = 0; k < 31; k++) {
-		if (data[start + 10 + 8 * k] > 0x0f)
+	for (i = 0; i < 31; i++) {
+		if (data[10 + 8 * i] > 0x0f)
 			return -1;
 	}
 
 	/* test #5 */
-	for (j = 0; j < 128; j++) {
-		if (data[start + 768 + j] > 0x7f)
+	for (i = 0; i < 128; i++) {
+		if (data[768 + i] > 0x7f)
 			return -1;
 	}
 
@@ -165,7 +162,7 @@ static int test_AC1D(uint8 *data, char *t, int s)
 
 const struct pw_format pw_ac1d = {
 	"AC1D Packer",
-	test_AC1D,
-	depack_AC1D
+	test_ac1d,
+	depack_ac1d
 };
 
