@@ -1,8 +1,9 @@
 /*
  * EurekaPacker.c   Copyright (C) 1997 Asle / ReDoX
- *		    Modified by Claudio Matsuoka
  *
  * Converts MODs packed with Eureka packer back to ptk
+ *
+ * Modified in 2006,2007,2014 by Claudio Matsuoka
  */
 
 #include <string.h>
@@ -86,115 +87,104 @@ static int depack_eu(FILE *in, FILE *out)
 	return 0;
 }
 
-static int test_eu (uint8 *data, char *t, int s)
+static int test_eu(uint8 *data, char *t, int s)
 {
-	int j, k, l, m, n, o;
-	int start = 0;
+	int i;
+	int len, max_pat, smp_offs;
+	int max_trk, min_trk;
 
-	PW_REQUEST_DATA (s, 1084);
+	PW_REQUEST_DATA(s, 1084);
 
 	/* test 2 */
-	j = data[start + 950];
-	if (j == 0 || j > 127)
+	len = data[950];
+	if (len == 0 || len > 127)
 		return -1;
 
 	/* test #3  finetunes & volumes */
-	for (k = 0; k < 31; k++) {
-		o = (data[start + 42 + k * 30] << 8) +
-			data[start + 43 + k * 30];
-		m = (data[start + 46 + k * 30] << 8) +
-			data[start + 47 + k * 30];
-		n = (data[start + 48 + k * 30] << 8) +
-			data[start + 49 + k * 30];
-		o *= 2;
-		m *= 2;
-		n *= 2;
-		if (o > 0xffff || m > 0xffff || n > 0xffff)
+	for (i = 0; i < 31; i++) {
+		uint8 *d = data + i * 30;
+		int size = readmem16b(d + 42) << 1;
+		int start = readmem16b(d + 46) << 1;
+		int lsize = readmem16b(d + 48) << 1;
+
+		if (size > 0xffff || start > 0xffff || lsize > 0xffff)
 			return -1;
 
-		if ((m + n) > (o + 2))
+		if (start + lsize > size + 2)
 			return -1;
 
-		if (data[start + 44 + k * 30] > 0x0f ||
-			data[start + 45 + k * 30] > 0x40)
+		if (d[44] > 0x0f || d[45] > 0x40)
 			return -1;
 	}
 
 
 	/* test 4 */
-	l = (data[start + 1080] << 24) + (data[start + 1081] << 16)
-		+ (data[start + 1082] << 8) + data[start + 1083];
+	smp_offs = readmem32b(data + 1080);
 
 #if 0
-	if ((l + start) > in_size)
+	if (smp_offs > in_size)
 		return -1;
 #endif
 
-	if (l < 1084)
+	if (smp_offs < 1084)
 		return -1;
 
-	m = 0;
 	/* pattern list */
-	for (k = 0; k < j; k++) {
-		n = data[start + 952 + k];
-		if (n > m)
-			m = n;
-		if (n > 127)
+	max_pat = 0;
+	for (i = 0; i < len; i++) {
+		int pat = data[952 + i];
+		if (pat > max_pat)
+			max_pat = pat;
+		if (pat > 127)
 			return -1;
 	}
-	k += 2;		/* to be sure .. */
-
-	while (k != 128) {
-		if (data[start + 952 + k] != 0)
+	for (i += 2; i < 128; i++) {
+		if (data[952 + i] != 0)
 			return -1;
-		k += 1;
 	}
-	m += 1;
-	/* m is the highest pattern number */
 
+	max_pat++;
 
 	/* test #5 */
-	/* j is still the size if the pattern table */
-	/* l is still the address of the sample data */
-	/* m is the highest pattern number */
-	n = 0;
-	j = 999999L;
+	/* max_trkptr is the highest track address */
+	/* min_trkptr is the lowest track address */
+	max_trk = 0;
+	min_trk = 999999;
 
-	PW_REQUEST_DATA (s, start + (m * 4) * 2 + 1085);
+	PW_REQUEST_DATA(s, max_pat * 4 * 2 + 1085);
 
-	for (k = 0; k < (m * 4); k++) {
-		o = (data[start + k * 2 + 1084] << 8) +
-			data[start + k * 2 + 1085];
-		if (o > l || o < 1084)
+	for (i = 0; i < (max_pat * 4); i++) {
+		int trk = readmem16b(data + i * 2 + 1084);
+		if (trk > smp_offs || trk < 1084)
 			return -1;
-		if (o > n)
-			n = o;
-		if (o < j)
-			j = o;
+		if (trk > max_trk)
+			max_trk = trk;
+		if (trk < min_trk)
+			min_trk = trk;
 	}
-	/* o is the highest track address */
-	/* j is the lowest track address */
 
 	/* test track datas */
 	/* last track wont be tested ... */
-	for (k = j; k < o; k++) {
-		if ((data[start + k] & 0xC0) == 0xC0)
+	for (i = min_trk; i < max_trk; i++) {
+		if ((data[i] & 0xc0) == 0xc0)
 			continue;
-		if ((data[start + k] & 0xC0) == 0x80) {
-			k += 2;
+
+		if ((data[i] & 0xc0) == 0x80) {
+			i += 2;
 			continue;
 		}
-		if ((data[start + k] & 0xC0) == 0x40) {
-			if ((data[start + k] & 0x3F) == 0x00 &&
-				data[start + k + 1] == 0x00)
+
+		if ((data[i] & 0xc0) == 0x40) {
+			if ((data[i] & 0x3f) == 0 && data[i + 1] == 0)
 				return -1;
-			k += 1;
+			i++;
 			continue;
 		}
-		if ((data[start + k] & 0xC0) == 0x00) {
-			if (data[start + k] > 0x13)
+
+		if ((data[i] & 0xc0) == 0) {
+			if (data[i] > 0x13)
 				return -1;
-			k += 3;
+			i += 3;
 			continue;
 		}
 	}
