@@ -1,9 +1,23 @@
 /* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
- * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU Lesser General Public License. See COPYING.LIB
- * for more information.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
 /* Ultimate Soundtracker support based on the module format description
@@ -11,9 +25,6 @@
  */
 
 #include <ctype.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-
 #include "loader.h"
 #include "mod.h"
 #include "period.h"
@@ -40,11 +51,11 @@ static int st_test(HIO_HANDLE *f, char *t, const int start)
     int pat, smp_size;
     struct st_header mh;
     uint8 mod_event[4];
-    struct stat st;
+    long size;
 
-    hio_stat(f, &st);
+    size = hio_size(f);
 
-    if (st.st_size < 600)
+    if (size < 600)
 	return -1;
 
     smp_size = 0;
@@ -117,7 +128,7 @@ static int st_test(HIO_HANDLE *f, char *t, const int start)
     if (smp_size < 8)
 	return -1;
 
-    if (st.st_size < (600 + pat * 1024 + smp_size))
+    if (size < (600 + pat * 1024 + smp_size))
 	return -1;
 
     for (i = 0; i < pat; i++) {
