@@ -1,9 +1,23 @@
 /* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
- * This file is part of the Extended Module Player and is distributed
- * under the terms of the GNU Lesser General Public License. See COPYING.LIB
- * for more information.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies or substantial portions of the Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+ * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+ * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+ * THE SOFTWARE.
  */
 
 #include "loader.h"
@@ -235,7 +249,7 @@ static int sym_load(struct module_data *m, HIO_HANDLE *f, const int start)
 	struct xmp_module *mod = &m->mod;
 	struct xmp_event *event;
 	int i, j;
-	int ver, infolen, sn[64];
+	int /*ver,*/ infolen, sn[64];
 	uint32 a, b;
 	uint8 *buf;
 	int size, ret;
@@ -245,7 +259,7 @@ static int sym_load(struct module_data *m, HIO_HANDLE *f, const int start)
 
 	hio_seek(f, 8, SEEK_CUR);	/* BASSTRAK */
 
-	ver = hio_read8(f);
+	/*ver =*/ hio_read8(f);
 	set_type(m, "Digital Symphony");
 
 	mod->chn = hio_read8(f);
