@@ -1,4 +1,4 @@
-/* Extended Module Player format loaders
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -131,10 +131,20 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 		    event->fxp = *pat++;
 		}
 
-		if (event->note == 0x61)
-		    event->note = XMP_KEY_OFF;
-		else if (event->note > 0)
+		if (event->note == 0x61) {
+		    /* See OpenMPT keyoff+instr.xm test case */
+		    if (event->fxt == 0x0e && (event->fxp >> 4) == 0x0d) {
+			if (event->ins) {
+				event->note = event->fxp = event->fxt = 0;
+			} else {
+				event->note = XMP_KEY_OFF;
+			}
+		    } else {
+		    	event->note = event->ins ? XMP_KEY_FADE : XMP_KEY_OFF;
+		    }
+		} else if (event->note > 0) {
 		    event->note += 12;
+		}
 
 		if (!event->vol)
 		    continue;
@@ -185,6 +195,27 @@ static int load_patterns(struct module_data *m, int version, HIO_HANDLE *f)
 		case 0x0f:	/* Tone portamento */
 		    event->f2t = FX_TONEPORTA;
 		    event->f2p = (event->vol - 0xf0) << 4;
+
+		    /* From OpenMPT TonePortamentoMemory.xm:
+		     * "Another nice bug (...) is the combination of both
+		     *  portamento commands (Mx and 3xx) in the same cell:
+		     *  The 3xx parameter is ignored completely, and the Mx
+		     *  parameter is doubled. (M2 3FF is the same as M4 000)
+		     */
+		    if (event->fxt == FX_TONEPORTA || event->fxt == FX_TONE_VSLIDE) {
+			if (event->fxt == FX_TONEPORTA) {
+			    event->fxt = 0;
+			} else {
+			    event->fxt = FX_VOLSLIDE;
+			}
+			event->fxp = 0;
+
+			if (event->f2p < 0x80) {
+				event->f2p <<= 1;
+			} else {
+				event->f2p = 0xff;
+			}
+		    }
 		    break;
 		}
 		event->vol = 0;
