@@ -1,4 +1,4 @@
-/* Extended Module Player core player
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -264,7 +264,7 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 	struct xmp_instrument *instrument;
 	int finalvol;
 	uint16 vol_envelope;
-	int gvol, end;
+	int gvol;
 
 	instrument = get_instrument(ctx, xc->ins);
 
@@ -281,8 +281,8 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 
 	if (TEST_NOTE(NOTE_FADEOUT | NOTE_RELEASE) || act == VIRT_ACTION_FADE
 	    || act == VIRT_ACTION_OFF) {
-		if (xc->fadeout > instrument->rls) {
-			xc->fadeout -= instrument->rls;
+		if (xc->fadeout > xc->ins_fade) {
+			xc->fadeout -= xc->ins_fade;
 		} else {
 			xc->fadeout = 0;
 			SET_NOTE(NOTE_END);
@@ -304,14 +304,14 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 		}
 	}
 
-	vol_envelope = get_envelope(&instrument->aei, xc->v_idx, 64, &end);
-	xc->v_idx = update_envelope(&instrument->aei, xc->v_idx, DOENV_RELEASE,
-						HAS_QUIRK(QUIRK_ENVSUS));
-	if (end) {
+	vol_envelope = get_envelope(&instrument->aei, xc->v_idx, 64);
+	if (check_envelope_end(&instrument->aei, xc->v_idx)) {
 		if (vol_envelope == 0)
 			SET_NOTE(NOTE_END);
 		SET_NOTE(NOTE_ENV_END);
 	}
+	xc->v_idx = update_envelope(&instrument->aei, xc->v_idx, DOENV_RELEASE,
+						HAS_QUIRK(QUIRK_ENVSUS));
 
 	/* If note ended in background channel, we can safely reset it */
 	if (TEST_NOTE(NOTE_END) && chn >= p->virt.num_tracks) {
@@ -380,7 +380,7 @@ static void process_volume(struct context_data *ctx, int chn, int t, int act)
 	else
 		finalvol = finalvol * p->smix_vol / 100;
 
-	xc->info_finalvol = finalvol;
+	xc->info_finalvol = TEST_NOTE(NOTE_SAMPLE_END) ? 0 :finalvol;
 
 	virt_setvol(ctx, chn, finalvol);
 }
@@ -401,11 +401,10 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 #ifndef LIBXMP_CORE_DISABLE_IT
 	int cutoff, resonance;
 #endif
-	int end;
 
 	instrument = get_instrument(ctx, xc->ins);
 
-	frq_envelope = get_envelope(&instrument->fei, xc->f_idx, 0, &end);
+	frq_envelope = get_envelope(&instrument->fei, xc->f_idx, 0);
 	xc->f_idx = update_envelope(&instrument->fei, xc->f_idx, DOENV_RELEASE,
 						HAS_QUIRK(QUIRK_ENVSUS));
 
@@ -458,7 +457,7 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 
 	/* Envelope */
 
-	if (~instrument->fei.flg & XMP_ENVELOPE_FLT) {
+	if (xc->f_idx >= 0 && (~instrument->fei.flg & XMP_ENVELOPE_FLT)) {
 		/* IT pitch envelopes are always linear, even in Amiga period
 		 * mode. Each unit in the envelope scale is 1/25 semitone.
 		 */
@@ -498,8 +497,10 @@ static void process_frequency(struct context_data *ctx, int chn, int t, int act)
 		return;
 	}
 
-	if (instrument->fei.flg & XMP_ENVELOPE_FLT) {
+	if (xc->f_idx >= 0 && (instrument->fei.flg & XMP_ENVELOPE_FLT)) {
 		cutoff = xc->filter.cutoff * frq_envelope >> 8;
+		if (cutoff > 0xfd)
+			cutoff = 0xfd;
 	} else {
 		cutoff = xc->filter.cutoff;
 	}
@@ -531,11 +532,10 @@ static void process_pan(struct context_data *ctx, int chn, int t, int act)
 	struct xmp_instrument *instrument;
 	int finalpan, panbrello = 0;
 	int pan_envelope;
-	int end;
 
 	instrument = get_instrument(ctx, xc->ins);
 
-	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32, &end);
+	pan_envelope = get_envelope(&instrument->pei, xc->p_idx, 32);
 	xc->p_idx = update_envelope(&instrument->pei, xc->p_idx, DOENV_RELEASE,
 						HAS_QUIRK(QUIRK_ENVSUS));
 
@@ -767,6 +767,12 @@ static void play_channel(struct context_data *ctx, int chn, int t)
 		}
         }
 
+	/* Do keyoff */
+	if (xc->keyoff) {
+		if (--xc->keyoff == 0)
+			SET_NOTE(NOTE_RELEASE);
+	}
+
 	process_volume(ctx, chn, t, act);
 	process_frequency(ctx, chn, t, act);
 	process_pan(ctx, chn, t, act);
@@ -775,12 +781,6 @@ static void play_channel(struct context_data *ctx, int chn, int t)
 	update_frequency(ctx, chn, t);
 	update_pan(ctx, chn, t);
 
-	/* Do keyoff */
-	if (xc->keyoff) {
-		if (--xc->keyoff == 0)
-			SET_NOTE(NOTE_RELEASE);
-	}
-
 	if (HAS_QUIRK(QUIRK_INVLOOP)) {
 		update_invloop(m, xc);
 	}
@@ -932,6 +932,23 @@ void player_set_fadeout(struct context_data *ctx, int chn)
 
 #endif
 
+static void update_from_ord_info(struct context_data *ctx)
+{
+	struct player_data *p = &ctx->p;
+	struct module_data *m = &ctx->m;
+	struct ord_data *oinfo = &m->xxo_info[p->ord];
+
+	if (oinfo->speed)
+		p->speed = oinfo->speed;
+	p->bpm = oinfo->bpm;
+	p->gvol = oinfo->gvl;
+	p->current_time = oinfo->time;
+	p->frame_time = m->time_factor * m->rrate / p->bpm;
+
+#ifndef LIBXMP_CORE_PLAYER
+	p->st26_speed = oinfo->st26_speed;
+#endif
+}
 
 int xmp_start_player(xmp_context opaque, int rate, int format)
 {
@@ -995,16 +1012,14 @@ int xmp_start_player(xmp_context opaque, int rate, int format)
 		f->end_point = p->scan[0].num;
 	}
 
-	p->gvol = m->xxo_info[p->ord].gvl;
-	p->bpm = m->xxo_info[p->ord].bpm;
-	p->speed = m->xxo_info[p->ord].speed;
-	p->frame_time = m->time_factor * m->rrate / p->bpm;
+	update_from_ord_info(ctx);
 
 	if (virt_on(ctx, mod->chn + smix->chn) != 0) {
 		ret = -XMP_ERROR_INTERNAL;
 		goto err;
 	}
 
+	f->delay = 0;
 	f->jumpline = 0;
 	f->jump = -1;
 	f->pbreak = 0;
@@ -1101,11 +1116,7 @@ int xmp_play_frame(xmp_context opaque)
 
 		next_order(ctx);
 
-		if (m->xxo_info[p->ord].speed)
-			p->speed = m->xxo_info[p->ord].speed;
-		p->bpm = m->xxo_info[p->ord].bpm;
-		p->gvol = m->xxo_info[p->ord].gvl;
-		p->current_time = m->xxo_info[p->ord].time;
+		update_from_ord_info(ctx);
 
 		virt_reset(ctx);
 		reset_channels(ctx);
@@ -1131,6 +1142,17 @@ int xmp_play_frame(xmp_context opaque)
 		}
 
 		read_row(ctx, mod->xxo[p->ord], p->row);
+
+#ifndef LIBXMP_CORE_PLAYER
+		if (p->st26_speed) {
+			if  (p->st26_speed & 0x10000) {
+				p->speed = (p->st26_speed & 0xff00) >> 8;
+			} else {
+				p->speed = p->st26_speed & 0xff;
+			}
+			p->st26_speed ^= 0x10000;
+		}
+#endif
 	}
 
 	inject_event(ctx);
@@ -1343,4 +1365,3 @@ void xmp_get_frame_info(xmp_context opaque, struct xmp_frame_info *info)
 		}
 	}
 }
-
