@@ -57,6 +57,7 @@ struct retrig_control {
 #define NOTE_END	(1 << 2)
 #define NOTE_CUT	(1 << 3)
 #define NOTE_ENV_END	(1 << 4)
+#define NOTE_SAMPLE_END	(1 << 5)
 
 #define IS_VALID_INSTRUMENT(x) ((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0)
 #define IS_VALID_INSTRUMENT_OR_SFX(x) (((uint32)(x) < mod->ins && mod->xxi[(x)].nsm > 0) || (smix->ins > 0 && (uint32)(x) < mod->ins + smix->ins))
@@ -83,15 +84,16 @@ struct channel_data {
 	int delay;		/* Note delay in frames */
 	int keyoff;		/* Key off counter */
 	int fadeout;		/* Current fadeout (release) value */
+	int ins_fade;		/* Instrument fadeout value */
 	int gliss;		/* Glissando active */
 	int volume;		/* Current volume */
 	int gvl;		/* Global volume for instrument for IT */
 	int offset;		/* Sample offset memory */
 	int offset_val;		/* Sample offset */
 
-	uint16 v_idx;		/* Volume envelope index */
-	uint16 p_idx;		/* Pan envelope index */
-	uint16 f_idx;		/* Freq envelope index */
+	int v_idx;		/* Volume envelope index */
+	int p_idx;		/* Pan envelope index */
+	int f_idx;		/* Freq envelope index */
 
 	struct {
 		struct lfo lfo;
@@ -223,7 +225,7 @@ struct channel_data {
 };
 
 
-void process_fx(struct context_data *, struct channel_data *, int, uint8, uint8, uint8, int);
+void process_fx(struct context_data *, struct channel_data *, int, struct xmp_event *, int);
 void filter_setup(int, int, int, int*, int*, int *);
 int read_event(struct context_data *, struct xmp_event *, int);
 
