@@ -1,4 +1,4 @@
-/* Extended Module Player core player
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,6 +26,7 @@
 #include "effects.h"
 #include "virtual.h"
 #include "period.h"
+#include "envelope.h"
 
 #ifndef LIBXMP_CORE_PLAYER
 #include "med_extras.h"
@@ -47,7 +48,7 @@ static struct xmp_subinstrument *get_subinstrument(struct context_data *ctx,
 	struct xmp_module *mod = &m->mod;
 	struct xmp_instrument *instrument;
 
-	if (ins >= 0 && ins < mod->ins) {
+	if (IS_VALID_INSTRUMENT(ins)) {
 		instrument = &mod->xxi[ins];
 		if (key >= 0 && key < XMP_MAX_KEYS) {
 			int mapped = instrument->map[key].ins;
@@ -62,22 +63,39 @@ static struct xmp_subinstrument *get_subinstrument(struct context_data *ctx,
 static void reset_envelopes(struct context_data *ctx, struct channel_data *xc,
 				int force_cut)
 {
+	struct module_data *m = &ctx->m;
+	struct xmp_module *mod = &m->mod;
 	struct xmp_instrument *xxi;
 
+	if (!IS_VALID_INSTRUMENT(xc->ins))
+		return;
+
+ 	RESET_NOTE(NOTE_ENV_END);
+
+	if (force_cut) {
+		xc->v_idx = 0;
+		xc->p_idx = 0;
+		xc->f_idx = 0;
+		return;
+	}
+
 	xxi = get_instrument(ctx, xc->ins);
 
 	/* Reset envelope positions */
-	if (force_cut || (~xxi->aei.flg & XMP_ENVELOPE_CARRY)) {
+	if (~xxi->aei.flg & XMP_ENVELOPE_CARRY) {
 		xc->v_idx = 0;
 	}
-	if (force_cut || (~xxi->pei.flg & XMP_ENVELOPE_CARRY)) {
+	if (~xxi->pei.flg & XMP_ENVELOPE_CARRY) {
 		xc->p_idx = 0;
 	}
-	if (force_cut || (~xxi->fei.flg & XMP_ENVELOPE_CARRY)) {
+
+	if (xxi->fei.flg & XMP_ENVELOPE_CARRY) {
+		if (check_envelope_end(&xxi->fei, xc->f_idx)) {
+			xc->f_idx = -1;
+		}
+	} else {
 		xc->f_idx = 0;
 	}
-
-	RESET_NOTE(NOTE_ENV_END);
 }
 
 static void set_effect_defaults(struct context_data *ctx, int note,
@@ -186,6 +204,7 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 				}
 			} else {
 				xc->ins = ins;
+				xc->ins_fade = mod->xxi[ins].rls;
 			}
 		} else {
 			new_invalid_ins = 1;
@@ -243,8 +262,8 @@ static int read_event_mod(struct context_data *ctx, struct xmp_event *e, int chn
 	}
 
 	/* Secondary effect handled first */
-	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
-	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
+	process_fx(ctx, xc, chn, e, 1);
+	process_fx(ctx, xc, chn, e, 0);
 	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL))
@@ -282,16 +301,39 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	int new_invalid_ins;
 	int is_toneporta;
 	int use_ins_vol;
+	int k00 = 0;
+	struct xmp_event ev;
+
+	memcpy(&ev, e, sizeof (struct xmp_event));
 
 	xc->flags = 0;
 	note = -1;
-	key = e->note;
-	ins = e->ins;
+	key = ev.note;
+	ins = ev.ins;
 	new_invalid_ins = 0;
 	is_toneporta = 0;
 	use_ins_vol = 0;
 
-	if (IS_TONEPORTA(e->fxt) || IS_TONEPORTA(e->f2t)) {
+	/* From the OpenMPT key_off.xm test case:
+	 * "Key off at tick 0 (K00) is very dodgy command. If there is a note
+	 *  next to it, the note is ignored. If there is a volume column
+	 *  command or instrument next to it and the current instrument has
+	 *  no volume envelope, the note is faded out instead of being cut."
+	 */
+	if (ev.fxt == FX_KEYOFF && ev.fxp == 0) {
+		k00 = 1;
+		key = 0;
+
+		if (ins || ev.vol || ev.f2t) {
+			if (IS_VALID_INSTRUMENT(xc->ins) &&
+			    ~mod->xxi[xc->ins].aei.flg & XMP_ENVELOPE_ON) {
+				SET_NOTE(NOTE_FADEOUT);
+				ev.fxt = 0;
+			}
+		}
+	}
+
+	if (IS_TONEPORTA(ev.fxt) || IS_TONEPORTA(ev.f2t)) {
 		is_toneporta = 1;
 	}
 
@@ -306,12 +348,15 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	}
 
 	/* Do this regardless if the instrument is invalid or not */
-	if (e->ins) {
+	if (ev.ins) {
 		SET(NEW_INS);
 		use_ins_vol = 1;
 		xc->fadeout = 0x10000;
 		xc->per_flags = 0;
-		RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
+		RESET_NOTE(NOTE_RELEASE);
+		if (!k00) {
+			RESET_NOTE(NOTE_FADEOUT);
+		}
 
 		if (IS_VALID_INSTRUMENT(ins - 1)) {
 			if (!is_toneporta)
@@ -333,32 +378,6 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 
 	/* Check note */
 
-	if (key) {
-		SET(NEW_NOTE);
-
-		if (key == XMP_KEY_OFF) {
-			SET_NOTE(NOTE_RELEASE);
-			use_ins_vol = 0;
-		} else if (is_toneporta) {
-			/* set key to 0 so we can have the tone portamento from
-			 * the original note (see funky_stars.xm pos 5 ch 9)
-			 */
-			key = 0;
-
-			/* And do the same if there's no keyoff (see comic
-			 * bakery remix.xm pos 1 ch 3)
-			 */
-		}
-
-		if (e->ins == 0 && !IS_VALID_INSTRUMENT(xc->old_ins - 1)) {
-			new_invalid_ins = 1;
-		}
-
-		if (new_invalid_ins) {
-			virt_resetchannel(ctx, chn);
-		}
-	}
-
 	/* FT2: Retrieve old instrument volume */
 	if (ins) {
 		struct xmp_subinstrument *sub;
@@ -371,6 +390,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			if (sub != NULL) {
 				xc->volume = sub->vol;
 				xc->pan.val = sub->pan;
+				xc->ins_fade = mod->xxi[xc->old_insvol].rls;
 				SET(NEW_VOL);
 			}
 		} else {
@@ -385,6 +405,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 			if (sub != NULL) {
 				xc->volume = sub->vol;
 				xc->pan.val = sub->pan;
+				xc->ins_fade = mod->xxi[xc->ins].rls;
 			} else {
 				xc->volume = 0;
 			}
@@ -393,15 +414,90 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 		}
 	}
 
-	if ((uint32)key <= XMP_MAX_KEYS && key > 0) {
+	if (key) {
+		SET(NEW_NOTE);
+
+		if (key == XMP_KEY_OFF) {
+			int env_on = 0;
+
+			/* OpenMPT NoteOffVolume.xm:
+			 * "If an instrument has no volume envelope, a note-off
+			 *  command should cut the sample completely - unless
+			 *  there is a volume command next it. This applies to
+			 *  both volume commands (volume and effect column)."
+			 */
+			if (IS_VALID_INSTRUMENT(xc->ins) &&
+			    (mod->xxi[xc->ins].aei.flg & XMP_ENVELOPE_ON)) {
+				env_on = 1;
+			}
+			
+			if (env_on || (ev.vol == 0 && ev.fxt != FX_VOLSET)) {
+				SET_NOTE(NOTE_RELEASE);
+				use_ins_vol = 0;
+			} else {
+				SET_NOTE(NOTE_FADEOUT);
+			}
+
+			/* See OpenMPT keyoff+instr.xm, pattern 2 row 0x40 */
+			if (env_on && ev.fxt == FX_EXTENDED &&
+			    (ev.fxp >> 4) == EX_DELAY) {
+				RESET_NOTE(NOTE_RELEASE);
+			}
+		} else if (key == XMP_KEY_FADE) {
+			/* Handle keyoff + instrument case (NoteOff2.xm) */
+			SET_NOTE(NOTE_FADEOUT);
+		} else if (is_toneporta) {
+			/* set key to 0 so we can have the tone portamento from
+			 * the original note (see funky_stars.xm pos 5 ch 9)
+			 */
+			key = 0;
+
+			/* And do the same if there's no keyoff (see comic
+			 * bakery remix.xm pos 1 ch 3)
+			 */
+		}
+
+		if (ev.ins == 0 && !IS_VALID_INSTRUMENT(xc->old_ins - 1)) {
+			new_invalid_ins = 1;
+		}
+
+		if (new_invalid_ins) {
+			virt_resetchannel(ctx, chn);
+		}
+	}
+
+
+	/* Check note range -- from the OpenMPT test NoteLimit.xm:
+	 * "I think one of the first things Fasttracker 2 does when parsing a
+	 *  pattern cell is calculating the “real” note (i.e. pattern note +
+	 *  sample transpose), and if this “real” note falls out of its note
+	 *  range, it is ignored completely (wiped from its internal channel
+	 *  memory). The instrument number next it, however, is not affected
+	 *  and remains in the memory."
+	 */
+	if (key > 0 && (uint32)key <= XMP_MAX_KEYS) {
+		int k = key - 1;
+		sub = get_subinstrument(ctx, xc->ins, k);
+		if (!new_invalid_ins && sub != NULL) {
+			int transp = mod->xxi[xc->ins].map[k].xpo;
+			int k2 = k + sub->xpo + transp;
+			if (k2 < 12 || k2 > 130) {
+				key = 0;
+				RESET(NEW_NOTE);
+			}
+		}
+	}
+
+	if (key > 0 && (uint32)key <= XMP_MAX_KEYS) {
 		xc->key = --key;
 		xc->fadeout = 0x10000;
 		RESET_NOTE(NOTE_END);
-		if (~mod->xxi[xc->ins].aei.flg & XMP_ENVELOPE_ON) {
-			RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
-		}
 
-		sub = get_subinstrument(ctx, xc->ins, key);
+		if (sub != NULL) {
+			if (~mod->xxi[xc->ins].aei.flg & XMP_ENVELOPE_ON) {
+				RESET_NOTE(NOTE_RELEASE|NOTE_FADEOUT);
+			}
+		}
 
 		if (!new_invalid_ins && sub != NULL) {
 			int transp = mod->xxi[xc->ins].map[key].xpo;
@@ -428,7 +524,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 
 	set_effect_defaults(ctx, note, sub, xc, is_toneporta);
 
-	if (ins && sub != NULL) {
+	if (ins && sub != NULL && !k00) {
 		/* Reset envelopes on new instrument, see olympic.xm pos 10
 		 * But make sure we have an instrument set, see Letting go
 		 * pos 4 chn 20
@@ -437,8 +533,8 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	}
 
 	/* Process new volume */
-	if (e->vol) {
-		xc->volume = e->vol - 1;
+	if (ev.vol) {
+		xc->volume = ev.vol - 1;
 		SET(NEW_VOL);
 		if (TEST_NOTE(NOTE_END)) {	/* m5v-nine.xm */
 			xc->fadeout = 0x10000;	/* OpenMPT NoteOff.xm */
@@ -450,8 +546,8 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 	xc->offset_val = 0;
 
 	/* Secondary effect handled first */
-	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
-	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
+	process_fx(ctx, xc, chn, &ev, 1);
+	process_fx(ctx, xc, chn, &ev, 0);
 	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL))
@@ -466,7 +562,7 @@ static int read_event_ft2(struct context_data *ctx, struct xmp_event *e, int chn
 
 		/* From the OpenMPT test cases (3xx-no-old-samp.xm):
 		 * "An offset effect that points beyond the sample end should
-		 * stop playback on this channel."
+		 *  stop playback on this channel."
 		 */
 
 		if (xc->offset_val >= mod->xxs[sub->sid].len) {
@@ -533,6 +629,7 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 				not_same_ins = 1;
 				if (!is_toneporta) {
 					xc->ins = ins;
+					xc->ins_fade = mod->xxi[ins].rls;
 				} else {
 					/* Get new instrument volume */
 					sub = get_subinstrument(ctx, ins, e->note);
@@ -608,8 +705,8 @@ static int read_event_st3(struct context_data *ctx, struct xmp_event *e, int chn
 	}
 
 	/* Secondary effect handled first */
-	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
-	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
+	process_fx(ctx, xc, chn, e, 1);
+	process_fx(ctx, xc, chn, e, 0);
 	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL))
@@ -702,6 +799,10 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 		is_toneporta = 0;
 	}
 
+	if (xc->period <= 0) {
+		is_toneporta = 0;
+	}
+
 	/* Check instrument */
 
 	if (ev.ins) {
@@ -836,6 +937,7 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 			reset_envelopes(ctx, xc, 1);
 		}
 		xc->ins = candidate_ins;
+		xc->ins_fade = mod->xxi[candidate_ins].rls;
 	}
 
 	sub = get_subinstrument(ctx, xc->ins, xc->key);
@@ -865,8 +967,8 @@ static int read_event_it(struct context_data *ctx, struct xmp_event *e, int chn)
 	/* According to Storlek test 25, Impulse Tracker handles the volume
 	 * column effects last.
 	 */
-	process_fx(ctx, xc, chn, ev.note, ev.fxt, ev.fxp, 0);
-	process_fx(ctx, xc, chn, ev.note, ev.f2t, ev.f2p, 1);
+	process_fx(ctx, xc, chn, &ev, 0);
+	process_fx(ctx, xc, chn, &ev, 1);
 	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL)) {
@@ -942,6 +1044,7 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 				}
 			} else {
 				xc->ins = ins;
+				xc->ins_fade = mod->xxi[ins].rls;
 			}
 		} else {
 			new_invalid_ins = 1;
@@ -1029,8 +1132,8 @@ static int read_event_med(struct context_data *ctx, struct xmp_event *e, int chn
 	}
 
 	/* Secondary effect handled first */
-	process_fx(ctx, xc, chn, e->note, e->f2t, e->f2p, 1);
-	process_fx(ctx, xc, chn, e->note, e->fxt, e->fxp, 0);
+	process_fx(ctx, xc, chn, e, 1);
+	process_fx(ctx, xc, chn, e, 0);
 	set_period(ctx, note, sub, xc, is_toneporta);
 
 	if (TEST(NEW_VOL)) {
@@ -1081,6 +1184,7 @@ static int read_event_smix(struct context_data *ctx, struct xmp_event *e, int ch
 	RESET_NOTE(NOTE_RELEASE);
 
 	xc->ins = ins;
+	xc->ins_fade = mod->xxi[ins].rls;
 
 	if (ins >= mod->ins && ins < mod->ins + smix->ins)
 		is_smix_ins = 1;
