@@ -1,4 +1,4 @@
-/* Extended Module Player core player
+/* Extended Module Player
  * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -65,6 +65,9 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
     struct xmp_event* event;
     int i, pat;
     struct ord_data *info;
+#ifndef LIBXMP_CORE_PLAYER
+    int st26_speed;
+#endif
 
     if (mod->len == 0)
 	return 0;
@@ -84,6 +87,9 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 
     speed = mod->spd;
     base_time = m->rrate;
+#ifndef LIBXMP_CORE_PLAYER
+    st26_speed = 0;
+#endif
 
     /* By erlk ozlr <erlk.ozlr@gmail.com>
      *
@@ -153,6 +159,10 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 	info->speed = speed;
 	info->time = clock + m->time_factor * alltmp / bpm;
 
+#ifndef LIBXMP_CORE_PLAYER
+	info->st26_speed = st26_speed;
+#endif
+
 	if (info->start_row == 0 && ord != 0) {
 	    if (ord == ep) {
 		clock_rst = clock + m->time_factor * alltmp / bpm;
@@ -258,6 +268,9 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		    if (parm) {
 			if (p->flags & XMP_FLAGS_VBLANK || parm < 0x20) {
 			    speed = parm;
+#ifndef LIBXMP_CORE_PLAYER
+			    st26_speed = 0;
+#endif
 			} else {
 			    clock += m->time_factor * alltmp / bpm;
 			    alltmp = 0;
@@ -273,6 +286,12 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		if (f2 == FX_SPEED_CP) {
 		    f2 = FX_S3M_SPEED;
 		}
+
+		/* ST2.6 speed processing */
+
+		if (f1 == FX_ICE_SPEED) {
+		    st26_speed = (MSN(p1) << 8) | LSN(p1);
+		}
 #endif
 
 		if ((f1 == FX_S3M_SPEED && p1) || (f2 == FX_S3M_SPEED && p2)) {
@@ -280,6 +299,9 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		    alltmp += cnt_row * speed * base_time;
 		    cnt_row  = 0;
 		    speed = parm;
+#ifndef LIBXMP_CORE_PLAYER
+		    st26_speed = 0;
+#endif
 		}
 
 		if ((f1 == FX_S3M_BPM && p1) || (f2 == FX_S3M_BPM && p2)) {
@@ -310,7 +332,7 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		            clock += m->time_factor * base_time / bpm;
 			}
 
-			// remove one row at final bpm
+			/* remove one row at final bpm */
 			clock -= m->time_factor * speed * base_time / bpm;
 
 		    } else if (MSN(parm) == 1) {
@@ -322,7 +344,7 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		            clock += m->time_factor * base_time / bpm;
 			}
 
-			// remove one row at final bpm
+			/* remove one row at final bpm */
 			clock -= m->time_factor * speed * base_time / bpm;
 
 		    } else {
@@ -379,6 +401,19 @@ static int scan_module(struct context_data *ctx, int ep, int chain)
 		row = loop_row[--loop_chn] - 1;
 		loop_chn = 0;
 	    }
+
+#ifndef LIBXMP_CORE_PLAYER
+	    if (st26_speed) {
+	        alltmp += cnt_row * speed * base_time;
+	        cnt_row  = 0;
+		if (st26_speed & 0x10000) {
+			speed = (st26_speed & 0xff00) >> 8;
+		} else {
+			speed = st26_speed & 0xff;
+		}
+		st26_speed ^= 0x10000;
+	    }
+#endif
 	}
 
 	if (break_row && pdelay) {
